@@ -23,8 +23,8 @@ except ImportError:
 
 def print_header():
     header = (
-        "optbinning (Version 0.6.1)\n"
-        "Copyright (c) 2019-2020 Guillermo Navas-Palencia, Apache License 2.0"
+        "optbinning (Version 0.9.0)\n"
+        "Copyright (c) 2019-2021 Guillermo Navas-Palencia, Apache License 2.0"
         "\n")
 
     print(header)
@@ -35,7 +35,12 @@ def print_optional_parameters(dict_default_options, dict_user_options):
     str_options = "  Begin options\n"
     for key, value in dict_default_options.items():
         user_value = dict_user_options[key]
-        user_flag = "d" if value == user_value else "U"
+
+        if (isinstance(user_value, (list, np.ndarray, dict)) or
+                value != user_value):
+            user_flag = "U"
+        else:
+            user_flag = "d"
 
         if user_value is None:
             user_value = "no"
@@ -161,7 +166,7 @@ def print_name_status(name, status):
 def print_main_info(name, status, time_total):
     print_name_status(name, status)
 
-    print("  Time    : {:<7.4f} sec\n".format(round(time_total, 4)))
+    print("  Time    : {:<7.4f} sec\n".format(time_total))
 
 
 def print_binning_information(binning_type, print_level, name, status,
