@@ -1,8 +1,10 @@
 from .scorecard import Scorecard
 from .plots import plot_auc_roc, plot_cap, plot_ks
+from .monitoring import ScorecardMonitoring
 
 
 __all__ = ["Scorecard",
+           "ScorecardMonitoring",
            "plot_auc_roc",
            "plot_cap",
            "plot_ks"]
